package br.com.cuidai.sdk.cuidai_plugin

import android.app.Activity
import android.os.Bundle
import android.util.Log
import androidx.annotation.NonNull
import androidx.core.net.toUri
import br.com.cuidai.app_components.extensions.bundleToMap
import br.com.cuidai.app_components.extensions.splitQuery
import br.com.cuidai.base_api.extensions.toBundle
import br.com.cuidai.router.Router.route
import br.com.cuidai.sdk.CuidaiSDK

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.lang.Exception
import java.net.URI
import java.net.URL

/** CuidaiPlugin */
class CuidaiPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
	/// The MethodChannel that will the communication between Flutter and native Android
	///
	/// This local reference serves to register the plugin with the Flutter Engine and unregister it
	/// when the Flutter Engine is detached from the Activity
	private lateinit var channelFromFlutter: MethodChannel
	private lateinit var channelToFlutter: MethodChannel

	private var activity: Activity? = null

	override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
		channelFromFlutter = MethodChannel(flutterPluginBinding.binaryMessenger, "cuidai_plugin")
		channelToFlutter = MethodChannel(flutterPluginBinding.binaryMessenger, "cuidai_plugin_native")
		channelFromFlutter.setMethodCallHandler(this)
	}

	override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
		when (call.method) {
			"initSDK" -> {
				val appName: String = (call.arguments as? HashMap<*, *>)?.get("appName")?.toString() ?: "vitat"
				activity?.let {
					CuidaiSDK.init(
						it.application,
						appName,
						null,
						true,
						onRoute = { uri, parameters, _ ->
							if (uri.contains("external")){
								try {
									splitQuery(uri)["route"]?.let { route ->
										val params = parameters.bundleToMap()
										activity?.runOnUiThread {
											channelToFlutter.invokeMethod(
													"route",
													mapOf("route" to route, "params" to params)
											)
										}
									}
								} catch (e: Exception){
									print(e)
								}
							}
						}
					)
				}
			}
			"getPlatformVersion" -> {
				result.success("Android ${android.os.Build.VERSION.RELEASE}")
			}
			"openChat" -> {
				activity?.route("default")
			}
			"setUserProperty" -> {
				val key: String? = (call.arguments as? HashMap<*, *>)?.get("key")?.toString()
				val value: String? = (call.arguments as? HashMap<*, *>)?.get("value")?.toString()
				key?.let { k ->
					CuidaiSDK.setUserProperty(k, value)
				}
			}
			"setUserProperties" -> {
				val properties = (call.arguments as? HashMap<*, *>)?.get("properties") as? Map<String,String?>
				properties?.let {
					CuidaiSDK.setUserProperties(it)
				}
			}
			"route" -> {
				val route: String? = (call.arguments as? HashMap<*, *>)?.get("route")?.toString()
				val params: HashMap<String, String>? = (call.arguments as? HashMap<*, *>)?.get("params") as? HashMap<String,String>
				route?.replace("flutter://", "")?.let {
					activity?.route(it, params = params?.toBundle() ?: Bundle())
				}
			}
			else -> {
				result.notImplemented()
			}
		}
	}

	override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
		channelFromFlutter.setMethodCallHandler(null)
	}

	override fun onAttachedToActivity(binding: ActivityPluginBinding) {
		this.activity = binding.activity
	}

	override fun onDetachedFromActivityForConfigChanges() {}

	override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {}

	override fun onDetachedFromActivity() {}
}
