import Flutter
import UIKit
import CuidaiSDK

private enum CuidaiCommand: String {
    case initSDK
    case openChat
    case test
    case route
}

extension URL {
    func valueOf(_ queryParamaterName: String) -> String? {
        guard let url = URLComponents(string: self.absoluteString) else { return nil }
        return url.queryItems?.first(where: { $0.name == queryParamaterName })?.value
    }
}

public class SwiftCuidaiPlugin: NSObject, FlutterPlugin {

    private var channelFromNative: FlutterMethodChannel? = nil

    public override init() {
        let window: UIWindow = ((UIApplication.shared.delegate?.window)!)!
        let controller : FlutterViewController = window.rootViewController as! FlutterViewController
        self.channelFromNative = FlutterMethodChannel(name: "cuidai_plugin_native", binaryMessenger: controller.binaryMessenger)
    }

  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "cuidai_plugin", binaryMessenger: registrar.messenger())
    let instance = SwiftCuidaiPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

    guard let command = CuidaiCommand(rawValue: call.method) else {
        return
    }

    switch command {

        case .initSDK:
            guard let map = call.arguments as? [String: Any],
                let appName = map["appName"] as? String
            else {
                return
            }

            Cuidai.instantiate(
                scheme: appName ?? "vitat",
                authenticator: { () -> String in
                    return "pvi8QPAnOjWV2qHSgiiOTiMrl3vHIHpO8JLrlInjxoYaUxN3I3o2FFWkAECsR3Mf"
                },
                subscriber: {
                    return true
                },
                tagger: { (tags) in
                    #if DEBUG
                    print("SDK tags \(tags)")
                    #endif
                },
                router: { (route) in
                    #if DEBUG
                    print("SDK route \(route)")
                    #endif
                    if route.contains("external") {
                        if let newURL = URL(string: route) {
                            let flutterRoute:String = newURL.valueOf("route") ?? "empty"
                            self.channelFromNative?.invokeMethod("route", arguments: ["route": flutterRoute])
                        }
                    }
                }
            )

        case .openChat:
            Cuidai.open("cuidai://chatbot")

        case .test:
            guard
                let map = call.arguments as? [String: Any],
                let route = map["route"] as? String
            else {
                return
            }
            self.channelFromNative?.invokeMethod("route", arguments: ["route": route])

        case .route:
            guard
                let map = call.arguments as? [String: Any],
                let route = map["route"] as? String
            else {
                return
            }
            Cuidai.open(route)

    }
  }
}
